package app

import (
	"context"
	"os"
	"os/signal"
	"syscall"

	"git.espin.casa/albert/cml04-falcon-system/bundle/service"
	"git.espin.casa/albert/logger"
	"github.com/spf13/cobra"
)

func Run(cmd *cobra.Command, args []string) {
	// read flags
	logLevel, _ := cmd.Flags().GetString("log-level")
	userName, _ := cmd.Flags().GetString("db-username")
	userPass, _ := cmd.Flags().GetString("db-password")
	dbHost, _ := cmd.Flags().GetString("db-host")
	dbPort, _ := cmd.Flags().GetInt("db-port")
	dbName, _ := cmd.Flags().GetString("db-name")
	natsHost, _ := cmd.Flags().GetString("nats-host")
	natsPort, _ := cmd.Flags().GetInt("nats-port")
	// setup logger
	log := logger.New(os.Stdout, logLevel)
	// log fields
	logFields := logger.LogFields{
		"db_username": userName,
		"db_password": "*****",
		"db_host":     dbHost,
		"db_port":     dbPort,
		"db_name":     dbName,
		"nats_host":   natsHost,
		"nats_port":   natsPort,
		"log_level":   logLevel,
	}
	// create service
	service, err := service.New(&service.DBConfig{
		Username: userName,
		Password: userPass,
		Host:     dbHost,
		Port:     dbPort,
		Name:     userName,
	}, &service.NATSConfig{
		Host: natsHost,
		Port: natsPort,
	})
	// handler error
	if err != nil {
		log.Error("create service failed", err, logFields)
		return
	}
	// create context
	ctx, cancel := context.WithCancel(context.Background())
	// cancel main context on exit
	defer cancel()
	// on exit close service
	defer service.Close(ctx)
	// info banner
	log.Info("started falcon bundle service", logFields)
	// wait signal to finish
	signal := WaitSignal()
	log.Info("signal received", logFields.Add(logger.LogFields{
		"signal": signal,
	}))
}

// WaitSignal catching exit signal
func WaitSignal() os.Signal {
	ch := make(chan os.Signal, 2)
	signal.Notify(
		ch,
		syscall.SIGINT,
		syscall.SIGQUIT,
		syscall.SIGTERM,
	)
	for {
		sig := <-ch
		switch sig {
		case syscall.SIGINT, syscall.SIGQUIT, syscall.SIGTERM:
			return sig
		}
	}
}
